@@ -94,7 +94,8 @@
                                collapse_method = c("stop", "composite", "average"), retain_original = TRUE,
                                intercor = .5, partial_intercor = FALSE,
                                construct_x = NULL, construct_y = NULL,
-                               measure_x = NULL, measure_y = NULL, data=NULL, es_metric=c("r", "d"), ma_method, ...) {
+                               measure_x = NULL, measure_y = NULL, moderator_names = NULL,
+                               data=NULL, es_metric=c("r", "d"), ma_method, ...) {
 
      if(!is.null(data)) {
           es_data <- data[,es_data]
@@ -118,9 +119,17 @@
           }else{
                measure_y <- NULL
           }
-     }
-     additions <- list(...)
+          
+          if(length(moderator_names$all) > 0){
+               moderators <- as.data.frame(as_tibble(data)[,moderator_names$all])
+          }else{
+               moderators <- NULL
+          }
 
+     }
+
+     additional_args <- list(...)
+     
      es_metric <- match.arg(es_metric, c("r", "d"))
 
      dup_IDs <- duplicated(sample_id) | duplicated(sample_id,fromLast=TRUE)
@@ -144,6 +153,27 @@
           }else{
                citekey_comp <- NULL
           }
+          
+          if(!is.null(moderators)){
+               moderators_comp_i <- (as_tibble(moderators)[i,])
+               moderators_comp <- (as_tibble(moderators)[1,])
+               
+               if(!is.null(moderator_names$cat))
+                    moderators_comp[,moderator_names$cat] <-
+                    apply(as.data.frame(moderators_comp_i[,moderator_names$cat]), 2, function(x){
+                         paste(sort(unique(as.character(x))), collapse = " & ")
+                    })
+               
+               if(!is.null(moderator_names$noncat))
+                    moderators_comp[,moderator_names$noncat] <- 
+                    apply(as.data.frame(moderators_comp_i[,moderator_names$noncat]), 2, function(x){
+                         mean(x, na.rm = TRUE)
+                    })
+               
+               moderators_comp <- as.data.frame(moderators_comp)
+          }else{
+               moderators_comp <- NULL
+          }
 
           if(ma_method != "bb"){
                art_out <- .consolidate_dependent_artifacts(n = es_data$n[i],
@@ -202,32 +232,61 @@
           if(collapse_method == "composite"){
                if(length(intercor) > 1) {
                     if(is.null(construct_x) & is.null(construct_y)) stop("Multiple intercorrelations provided without effect-size construct labels.\nProvide either a scalar intercorrelation or effect size construct labels.")
-                    intercor_x <- intercor[construct_x[i][1]]
-                    intercor_y <- intercor[construct_y[i][1]]
+                    
+                    intercor_x <- intercor[paste(sample_id[i][1], construct_x[i][1])]
+                    intercor_y <- intercor[paste(sample_id[i][1], construct_y[i][1])]
+                    
+                    if(is.na(intercor_x)) intercor_x <- intercor[construct_x[i][1]]
+                    if(is.na(intercor_y)) intercor_y <- intercor[construct_y[i][1]]
+                    
+                    if(is.na(intercor_x) & is.na(intercor_y)){
+                         warning("Valid same-construct intercorrelations for constructs '", as.character(construct_x[i][1]), 
+                                 "' and '", as.character(construct_y[i][1]),
+                                 "' not provided for sample '", sample_id[i][1], 
+                                 "': '\n    Computing averages rather than composites", call. = FALSE)
+                    }else if(is.na(intercor_x) | is.na(intercor_y)){
+                         if(is.na(intercor_x)){
+                              warning("Valid same-construct intercorrelations for construct '", as.character(construct_x[i][1]), 
+                                      "' not provided for sample '", sample_id[i][1], 
+                                      "': '\n     Compositing using information from construct '", as.character(construct_y[i][1]), "' only", call. = FALSE)     
+                         }else{
+                              warning("Valid same-construct intercorrelations for construct '", as.character(construct_y[i][1]), 
+                                      "' not provided for sample '", sample_id[i][1], 
+                                      "': '\n     Compositing using information from construct '", as.character(construct_x[i][1]), "' only", call. = FALSE)     
+                         }
+                            
+                    }
+
                } else {
                     intercor_x <- intercor_y <- intercor
                }
 
-               if(length(partial_intercor) > 1) {
+               if(length(partial_intercor) > 1){
                     if(is.null(construct_y)) stop("Multiple intercorrelations provided without effect-size construct labels.\nProvide either a scalar intercorrelation or effect size construct labels.")
-                    intercor_y <- partial_intercor[construct_y[i][1]]
+                    partial_y <- partial_intercor[construct_y[i][1]]
+                    
+                    partial_y <- partial_y[paste(sample_id[i][1], construct_y[i][1])]
+
+                    if(is.na(partial_y)) partial_y <- partial_y[construct_y[i][1]]
                } else {
                     partial_y <- partial_intercor
-                    if(partial_y){
-                         if(!is.null(additions$.dx_internal_designation)){
-                              intercor_y <- mix_r_2group(rxy = intercor_y, dx = es_data$d, dy = es_data$d, p = es_data$pi)
-                              partial_y <- FALSE
-                         }
+               }
+               
+               if(partial_y){
+                    if(!is.null(additional_args$.dx_internal_designation)){
+                         intercor_y <- mix_r_2group(rxy = intercor_y, dx = es_data$d, dy = es_data$d, p = es_data$pi)
+                         partial_y <- FALSE
                     }
                }
 
-               if(is.null(measure_x) & is.null(measure_y)){
+               if(is.null(measure_x) & is.null(measure_y) & (!is.na(intercor_x) | !is.na(intercor_y))){
                     if(es_metric=="r") {
                          es_comp <- composite_r_scalar(mean_rxy = wt_mean(x = es_data$rxyi[i], wt = es_data$n_adj[i]),
-                                                       k_vars_x = length(es_data$rxyi[i]), mean_intercor_x = mean(c(intercor_x,intercor_y)),
+                                                       k_vars_x = length(es_data$rxyi[i]), mean_intercor_x = mean(c(intercor_x,intercor_y), na.rm = TRUE),
                                                        k_vars_y = 1, mean_intercor_y = intercor_y)
                     } else {
-                         es_comp <- composite_d_scalar(mean_d = wt_mean(x = es_data$d[i], wt = es_data$n_adj[i]), k_vars = length(es_data$dxyi[i]), mean_intercor = intercor_y, partial_intercor = partial_y)
+                         es_comp <- composite_d_scalar(mean_d = wt_mean(x = es_data$d[i], wt = es_data$n_adj[i]), k_vars = length(es_data$dxyi[i]), 
+                                                       mean_intercor = intercor_y, partial_intercor = partial_y)
                     }
 
                     if(ma_method != "bb"){
@@ -236,7 +295,7 @@
                          ryy_comp <- wt_mean(x = data_y$ryy[i], wt = es_data$n[i])
                          uy_comp  <- wt_mean(x = data_y$uy[i], wt = es_data$n[i])
                     }
-               } else if(!is.null(measure_x) & is.null(measure_y)) {
+               } else if(!is.null(measure_x) & is.null(measure_y) & !is.na(intercor_x)) {
                     if(es_metric=="r") {
                          es_comp <- composite_r_scalar(mean_rxy = wt_mean(x = es_data$rxyi[i], wt = es_data$n_adj[i]),
                                                        k_vars_x = length(es_data$rxyi[i]),  mean_intercor_x = intercor_x,
@@ -259,7 +318,7 @@
                     }
 
 
-               } else if(is.null(measure_x) & !is.null(measure_y)) {
+               } else if(is.null(measure_x) & !is.null(measure_y) & !is.na(intercor_y)) {
                     if(es_metric=="r") {
                          es_comp <- composite_r_scalar(mean_rxy = wt_mean(x = es_data$rxyi[i], wt = es_data$n_adj[i]),
                                                        k_vars_x = 1, mean_intercor_x = intercor_x,
@@ -274,7 +333,7 @@
                          ux_comp  <- wt_mean(x = data_x$ux[i], wt = es_data$n[i])
                     }
 
-               } else {
+               } else if(!is.null(measure_x) & !is.null(measure_y) & !is.na(intercor_x) & !is.na(intercor_y)){
                     kx <- length(unique(measure_x[i]))
                     ky <- length(unique(measure_y[i]))
 
@@ -298,16 +357,35 @@
                               ux_comp  <- wt_mean(x = data_x$ux[i], wt = es_data$n[i])
                          }
                     }
+               }else{
+                    if(es_metric=="r"){
+                         es_comp <- wt_mean(x = es_data$rxyi[i], wt = es_data$n_adj[i])
+                    }else{
+                         es_comp <- wt_mean(x = es_data$dxyi[i], wt = es_data$n_adj[i])
+                    }
+                    
+                    if(ma_method != "bb"){
+                         rxx_comp <- wt_mean(x = data_x$rxx[i], wt = es_data$n[i])
+                         ux_comp  <- wt_mean(x = data_x$ux[i], wt = es_data$n[i])
+                         ryy_comp <- wt_mean(x = data_y$ryy[i], wt = es_data$n[i])
+                         uy_comp  <- wt_mean(x = data_y$uy[i], wt = es_data$n[i])
+                    }
                }
           }
+          
+          if(ma_method != "bb"){
+               k_items_x_comp <- wt_mean(x = data_x$k_items_x[i], wt = es_data$n[i])
+               k_items_y_comp  <- wt_mean(x = data_y$k_items_y[i], wt = es_data$n[i])
+          }
 
           if(ma_method == "bb") rxx_comp <- ux_comp <- ryy_comp <- uy_comp <-
                rxx_restricted_comp <- ryy_restricted_comp <- ux_observed_comp <- uy_observed_comp <-
-               correct_rr_x <- correct_rr_y <- indirect_rr_x <- indirect_rr_y <- NULL
+               correct_rr_x <- correct_rr_y <- indirect_rr_x <- indirect_rr_y <- 
+                    k_items_x_comp <- k_items_y_comp <- NULL
 
           n_comp <- wt_mean(x = es_data$n[i], wt = es_data$n_adj[i])
           n_adj_comp <- wt_mean(x = es_data$n_adj[i], wt = es_data$n_adj[i])
-
+          
           if(all(c("d", "n1", "n2", "pi", "pa") %in% colnames(es_data))){
                n1_comp <- wt_mean(x = es_data$n1[i], wt = es_data$n_adj[i])
                n2_comp <- wt_mean(x = es_data$n2[i], wt = es_data$n_adj[i])
@@ -319,6 +397,7 @@
           }
 
           out <- list(sample_id = sample_id[i][1],
+                      moderators_comp = moderators_comp, 
                       es_comp = es_comp, n_comp = n_comp, n_adj_comp = n_adj_comp,
                       rxx_comp = rxx_comp, ryy_comp = ryy_comp,
                       ux_comp = ux_comp, uy_comp = uy_comp,
@@ -326,7 +405,9 @@
                       ryy_restricted_comp = ryy_restricted_comp,
                       ux_observed_comp = ux_observed_comp,
                       uy_observed_comp = uy_observed_comp,
-
+                      k_items_x_comp = k_items_x_comp, 
+                      k_items_y_comp = k_items_y_comp, 
+                      
                       correct_rr_x = correct_rr_x[i][1],
                       correct_rr_y = correct_rr_y[i][1],
 
@@ -382,6 +463,12 @@
 
           out
      })
+     
+     if(!is.null(moderators)){
+          mod_out <- as.data.frame(data.table::rbindlist(lapply(out, function(x) x$moderators_comp)))
+     }else{
+          mod_out <- NULL
+     }
 
      es_data_list <- list(construct_x = unlist(lapply(out, function(x) x$construct_x)),
                           construct_y = unlist(lapply(out, function(x) x$construct_y)),
@@ -399,7 +486,8 @@
      data_x_list <- list(rxx = unlist(lapply(out, function(x) x$rxx_comp)),
                          rxx_type = unlist(lapply(out, function(x) x$rxx_type)),
                          rxx_consistency = unlist(lapply(out, function(x) x$rxx_consistency)),
-
+                         k_items_x = unlist(lapply(out, function(x) x$k_items_x_comp)),
+                         
                          ux = unlist(lapply(out, function(x) x$ux_comp)),
                          rxx_restricted = unlist(lapply(out, function(x) x$rxx_restricted_comp)),
                          ux_observed = unlist(lapply(out, function(x) x$ux_observed_comp)),
@@ -413,7 +501,8 @@
      data_y_list <- list(ryy = unlist(lapply(out, function(x) x$ryy_comp)),
                          ryy_type = unlist(lapply(out, function(x) x$ryy_type)),
                          ryy_consistency = unlist(lapply(out, function(x) x$ryy_consistency)),
-
+                         k_items_y = unlist(lapply(out, function(x) x$k_items_y_comp)),
+                         
                          uy = unlist(lapply(out, function(x) x$uy_comp)),
                          ryy_restricted = unlist(lapply(out, function(x) x$ryy_restricted_comp)),
                          uy_observed = unlist(lapply(out, function(x) x$uy_observed_comp)),
@@ -429,6 +518,7 @@
      for(i in names(data_y_list)) if(is.null(data_y_list[[i]])) data_y_list[[i]] <- NULL
 
      es_data <- as.data.frame(es_data_list)
+     if(!is.null(mod_out)) es_data <- cbind(es_data, mod_out)
      data_x <- as.data.frame(data_x_list)
      data_y <- as.data.frame(data_y_list)
 
